/********************************************************************************
    motioncomptest - real-time motion compensation test
 ********************************************************************************
    LibVideoCoding - video coding library
    Copyright (C) 2002  Dirk Farin

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************************/

#include <libvideogfx/graphics/fileio/v4l.hh>
#include <libvideogfx/x11/imgwin.hh>
#include <libvideocoding/motioncomp/motioncomp.hh>

#include <iostream>
using namespace std;
using namespace videocoding;

const int width=352;
const int height=288;

int main()
{
  ImageWindow_Autorefresh_X11 win;
  win.Create(width,height,"test");

  V4L_Grabber grabber;
  grabber.SetResolution(width,height);
  grabber.DoChromaAvgOn422To420(true);
  //grabber.SetGreyscaleMode(true);
  grabber.StartGrabbing();

  MotionCompensation* mc = GetMotionCompensation(16,false,HalfPelDist,false);

  cout << "MC: " << mc->AskName() << endl;

  Image<Pixel> img;

  grabber.Grab(img);
  Image<Pixel> img2 = img.Clone();

  double w=0.0;

  for (;;)
    {
      grabber.Grab(img);

      MotionVector mv;

      mv.v = DoubleToFixed16_16(sin(w*3.141/180.0)*16.0);
      mv.h = DoubleToFixed16_16(cos(w*3.141/180.0)*16.0);

      mv.LimitResolution(HalfPelDist);

      for (int y=16;y<height-16;y+=16)
	for (int x=16;x<width-16;x+=16)
	  {
	    mc->Compensate(img2.AskBitmapY(),
			   img.AskBitmapY(),
			   x,y, mv, 16);

	    MotionVector mv2;
	    mv2.h = mv.h/2;
	    mv2.v = mv.v/2;

	    mc->Compensate(img2.AskBitmapU(),
			   img.AskBitmapU(),
			   x/2,y/2, mv2, 8);

	    mc->Compensate(img2.AskBitmapV(),
			   img.AskBitmapV(),
			   x/2,y/2, mv2, 8);
	  }

      win.Display(img2);

      w += 8.0;
      if (w>=360.0) w-=360.0;
    }

  return 0;
}
