/********************************************************************************
    LibVideoCoding - video coding library
    Copyright (C) 2002  Dirk Farin

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************************/

#include "libvideocoding/dct/fdct_fp_aan.hh"
#include <math.h>


namespace videocoding {

#define NC_COS1SQRT2 1.38703984532214746182161919156644 //cos(1*pi/16)*sqrt(2)
#define NC_COS2SQRT2 1.30656296487637652785664317342719 //cos(2*pi/16)*sqrt(2)
#define NC_COS3SQRT2 1.17587560241935871697446710461126 //cos(3*pi/16)*sqrt(2)
#define NC_COS5SQRT2 0.785694958387102181277897367657217//cos(5*pi/16)*sqrt(2)
#define NC_COS6SQRT2 0.541196100146196984399723205366389//cos(6*pi/16)*sqrt(2)
#define NC_COS7SQRT2 0.275899379282943012335957563669373//cos(7*pi/16)*sqrt(2)

  const double aansf[8] = {
    1.0, 
    NC_COS1SQRT2,
    NC_COS2SQRT2,
    NC_COS3SQRT2,
    1.0,  // cos(4*pi/16) * sqrt(2) = 1.0 exactly
    NC_COS5SQRT2,
    NC_COS6SQRT2,
    NC_COS7SQRT2
  };


  double aanscales[8][8];
  double costab[8 /* x */][8 /* u */];

#ifndef PI
#define PI 3.1415926535
#endif

  void Init_fdct()
  {
    int i, j;

    for (i = 0; i < 8; i++)
      for (j = 0; j < 8; j++)
	aanscales[i][j] = 1.0 / (aansf[i] * aansf[j] * 8.0);

    for (int x=0;x<8;x++)
      for (int u=0;u<8;u++)
	costab[x][u] = cos((2*x+1)*u*PI/16.0);
  }

}
