/********************************************************************************
  $Header: /home/farin/cvs/libvideocoding/libvideocoding/motioncomp/motioncomp.hh,v 1.6 2002/05/12 21:41:47 farin Exp $

    Low-level motion compensation routines.
 ********************************************************************************
    Copyright (C) 2002 Dirk Farin (see the README for complete list of authors)

    This program is distributed under GNU Public License (GPL) as
    outlined in the COPYING file that comes with the source distribution.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ********************************************************************************/

#ifndef LIBVIDEOCODING_MOTIONCOMP_MOTIONCOMP_HH
#define LIBVIDEOCODING_MOTIONCOMP_MOTIONCOMP_HH

#include <libvideogfx/graphics/datatypes/bitmap.hh>
#include <libvideocoding/datatypes/mv.hh>


namespace videocoding {
  using namespace videogfx;

  class MotionCompensation
  {
  public:
    virtual ~MotionCompensation() { }

    /* Create a prediction from the block located in "ref" at position ref_*+mv.
       Store the predicted block at "dest". "dest" need net be a complete image.
       It can also be a shorter memory segment of the size of a single block. */
    virtual void Compensate(Pixel* dest,int dest_stride,
			    const Bitmap<Pixel>& ref,
			    int ref_x0,int ref_y0,
			    const MotionVector& mv,int blkh) = 0;

    /* Create a prediction and store it to the position x0/y0 in the destination bitmap. */
    void Compensate(Bitmap<Pixel>& dest,
		    const Bitmap<Pixel>& ref,
		    int x0,int y0,
		    const MotionVector& mv,int blkh)
    {
      Compensate(&dest.AskFrame()[y0][x0],dest.AskStride(),
		 ref,x0,y0,mv,blkh);
    }

    virtual const char* AskName() const = 0;
  };



  MotionCompensation* GetMotionCompensation(int blkw,              // block width
					    bool avg,              // do averaging with destination
					    fixed16_16 resolution, // supported MV accuracy
					    bool accurate);        // accurate calculation vs. fast calculation

}

#endif
